package net.mc_pandacraft.java.plugin.pandacraftutils.commands;

import java.util.ArrayList;
import java.util.List;

import net.mc_pandacraft.java.plugin.pandacraftutils.modules.protection.TamedEntityProtectManager;
import net.mc_pandacraft.java.plugin.pandacraftutils.players.OnlinePlayer;
import net.mc_pandacraft.java.plugin.pandacraftutils.players.OnlinePlayerManager;
import net.mc_pandacraft.java.util.StringUtil;
import net.minecraft.server.v1_7_R1.EntityHorse;
import net.minecraft.server.v1_7_R1.GenericAttributes;

import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftHorse;
import org.bukkit.entity.Horse;
import org.bukkit.entity.Player;
import org.bukkit.entity.Tameable;

import com.google.common.collect.ImmutableList;

public class CommandAnimal extends AbstractCommandExecutor {
	
	public CommandAnimal() {
		super("animal");
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		
		if (!(sender instanceof Player)) {
			sender.sendMessage(ChatColor.RED+"seul un joueur en ligne peut effectuer cette commande");
			return true;
		}
		
		Player p = (Player) sender;
		OnlinePlayer op = OnlinePlayerManager.get(p);
		
		if (!op.isAnimalSelectionEnabled()) {
			op.enableAnimalSelection();
			p.sendMessage(ChatColor.GREEN+"Faites un clic droite pour sélectionner un animal apprivoisé (cheval, loup ou chat)");
			return true;
		}
		// la sélection d'un animal est déjà activée
		
		
		// si le joueur veut désactiver la sélection d'un animal
		if (args.length >= 1 && args[0].equalsIgnoreCase("annuler")) {
			p.sendMessage(ChatColor.GREEN+"La sélection d'un animal est désactivée");
			op.resetAnimalSelection();
			return true;
		}
		
		
		// si je joueur n'a pas encore sélectionné d'animal
		if (op.getSelectedAnimal() == null) {
			p.sendMessage(ChatColor.RED+"Faites d'abord un clic droit sur un animal apprivoisé (cheval, loup ou chat)");
			p.sendMessage(ChatColor.RED+"Pour désactiver la sélection, faites "+ChatColor.GRAY+"/animal annuler");
			return true;
		}

		TamedEntityProtectManager tepm = plugin.tamedEntityProtectManager;
		Tameable animal = op.getSelectedAnimal();
		String animalName = tepm.get_thisAnimal_Name(animal);
		
		if (!op.hasPermission("pandacraft.animal.admin")) {
			if (animal.getOwner() == null) {
				p.sendMessage(ChatColor.RED+animalName+ChatColor.RED+" n'appartient à personne");
				return true;
			}
			if (!animal.getOwner().equals(p)) {
				p.sendMessage(ChatColor.RED+animalName+ChatColor.RED+" n'est pas à vous");
				return true;
			}
		}
		
		
		
		
		if (args.length >= 2 && args[0].equalsIgnoreCase("donner")) {
			
			Player newOwner = plugin.getServer().getPlayer(args[1]);
			
			if (newOwner == null) {
				p.sendMessage(ChatColor.RED+"Le joueur indiqué n'est pas en ligne");
				return true;
			}
			
			animal.setOwner(newOwner);
			p.sendMessage(ChatColor.GREEN+"Vous venez de donner "+animalName+ChatColor.GREEN+" à "+newOwner.getDisplayName());
			op.resetAnimalSelection();
			
			return true;
		}
		
		if (args.length >= 2 && args[0].equalsIgnoreCase("modifier")) {
			
			if (!op.hasPermission("pandacraft.animal.modify")) {
				p.sendMessage(ChatColor.RED+"Vous n'avez pas la permission de modifier les attributs de l'animal");
				return true;
			}
		
			if (args.length >= 3 && args[1].equalsIgnoreCase("hauteur_saut")) {
			
				if (!(animal instanceof Horse)) {
					p.sendMessage(ChatColor.RED+"Vous ne pouvez pas modifier la hauteur de saut de cet animal");
					return true;
				}
				Horse h = (Horse)animal;
				
				try {
					double jump = Double.parseDouble(args[2]);
					p.sendMessage(ChatColor.RED+"Puissance de saut du cheval défini à "+StringUtil.formatDouble(jump));
					jump *= 0.2D;
					h.setJumpStrength(jump);
					
				} catch (NumberFormatException e) {
					p.sendMessage(ChatColor.RED+"La valeur indiquée est invalide");
					return true;
				}
				
			}
			if (args.length >= 3 && args[0].equalsIgnoreCase("vitesse")) {

				if (!(animal instanceof Horse)) {
					p.sendMessage(ChatColor.RED+"Vous ne pouvez pas modifier la vitesse de cet animal");
					return true;
				}
				
				Horse h = (Horse)animal;
				EntityHorse horse = ((CraftHorse)h).getHandle();
				
				try {
					double speed = Double.valueOf(args[1]).doubleValue();
					p.sendMessage(ChatColor.RED+"Vitesse du cheval défini à "+StringUtil.formatDouble(speed));
					speed *= 0.1D;
					horse.getAttributeInstance(GenericAttributes.d).setValue(speed);
					
				} catch (NumberFormatException e) {
					p.sendMessage(ChatColor.RED+"La valeur indiquée est invalide");
					return true;
				}
				
			}
		}
		
		
		
		
		return false;
	}
	
	
	

	public static final List<String> SECOND_ARG_MODIFIER = ImmutableList.of("hauteur_saut", "vitesse");
	
	
	@Override
	public List<String> onTabComplete(CommandSender sender, Command cmd, String alias, String[] args) {
		
		
		if (!(sender instanceof Player)) {
			return NO_PROPOSAL;
		}
		
		Player p = (Player) sender;
		OnlinePlayer op = OnlinePlayerManager.get(p);
		
		if (!op.isAnimalSelectionEnabled()) {
			return NO_PROPOSAL;
		}
		// la sélection d'un animal est déjà activée
		List<String> proposal_first_arg = new ArrayList<String>();
		
		proposal_first_arg.add("annuler");
		

		if (op.getSelectedAnimal() != null) {
			proposal_first_arg.add("donner");
			proposal_first_arg.add("modifier");
		}
		
		
		
		
		if (args.length == 1) {
			return getTabProposalFromToken(args[0], proposal_first_arg);
		}
		else if (args.length > 1 && op.getSelectedAnimal() != null) {
				if (args[0].equalsIgnoreCase("donner"))
					if (args.length == 2)
						return null;
				if (args[0].equalsIgnoreCase("modifier")) {
					if (args.length == 2)
						return getTabProposalFromToken(args[1], SECOND_ARG_MODIFIER);
				}
		}
		
		
		
		return NO_PROPOSAL;
	}

}
